##
#   Copyright (c) 2021 Valentin Weber
#
#   This file is part of the software steam-scheduled-download-killer.
#
#   The software is licensed under the European Union Public License
#   (EUPL) version 1.2 or later. You should have received a copy of
#   the english license text with the software. For your rights and
#   obligations under this license refer to the file LICENSE or visit
#   https://joinup.ec.europa.eu/community/eupl/og_page/eupl to view
#   official translations of the licence in another language of the EU.
##

"""Module for interacting with config."""

import os
from typing import IO, List, Union

from .core.models import SteamLibrary
from .core.utils import get_abspath

CONFIG_PREFIX = """\
##
#   This file was generated by steam-scheduled-download-killer (ssdk).
#
#   Each line contains the absolute path to a Steam Library Folder. You can
#   either use the ssdk command line interface to add additional folders or add
#   them manually by writing a new line in this file. Be aware that the
#   existence of the folder is only validated when using the command line
#   interface, so make sure the path is correct when adding it manually.
##
"""


class ConfigFileError(Exception):
    """Exception for errors during config file interaction."""


class Config:
    """Handler for interacting with files listing Steam Libraries."""

    def __init__(self, file: str) -> None:
        """Initialize handler for given config file."""
        self.file = file

    def read(self) -> List[SteamLibrary]:
        """Read list of Steam Libraries from config file."""
        return [SteamLibrary(lib) for lib in self._read()]

    def write(
        self,
        lib_dirs: Union[str, List[str]],
        overwrite_existing: bool = True
    ) -> None:
        """Write list of Steam Library Directories to config file."""
        if os.path.isfile(self.file) and not overwrite_existing:
            raise ConfigFileError("config file already exists", self.file)
        libs = []
        for lib in self._get_lib_dir_list(lib_dirs):
            lib_path = get_abspath(lib)
            if not os.path.isdir(lib_path):
                print(f"'{lib_path}' is not a directory")
            else:
                libs.append(lib_path)
                print(f"Adding '{lib_path}' to config")
        if libs:
            self._write(libs)
            print(f"Created config file at '{self.file}'")

    def remove(self, lib_dirs: Union[str, List[str]]):
        """Remove Steam Library Directory from config file."""
        cfg_libs = self._read()
        lib_dirs = self._get_lib_dir_list(lib_dirs)
        for lib in lib_dirs:
            lib_path = get_abspath(lib)
            if lib_path not in cfg_libs:
                print(f"Unable to find '{lib_path}' in config")
            else:
                cfg_libs.remove(lib_path)
                print(f"Removing '{lib_path}' from config")
        self._write(cfg_libs)

    def add(self, lib_dirs: Union[str, List[str]]) -> None:
        """Add Steam Library Directory to config file."""
        cfg_libs = self._read()
        lib_dirs = self._get_lib_dir_list(lib_dirs)
        for lib in lib_dirs:
            lib_path = get_abspath(lib)
            if lib_path in cfg_libs:
                print(f"'{lib_path}' is already in config")
            elif not os.path.isdir(lib_path):
                print(f"'{lib_path}' is not a directory")
            else:
                cfg_libs.append(lib_path)
                print(f"Adding '{lib_path}' to config")
        self._write(cfg_libs)

    def _open(self, filemode="r", encoding="utf-8") -> IO:
        """Open config file with specified mode and encoding."""
        try:
            return open(self.file, filemode, encoding=encoding)
        except OSError as exc:
            raise ConfigFileError("cannot access config", self.file) from exc

    def _read(self) -> List[str]:
        """Read list of Steam Library Directories from config file."""
        if not os.path.isfile(self.file):
            raise ConfigFileError("config file doesn't exist", self.file)
        with self._open() as cfg_file:
            lib_dirs = [
                line.strip() for line in cfg_file.readlines()
                if line.strip() and not line.startswith("#")
            ]
        return lib_dirs

    def _write(self, lib_dirs: List[str]):
        """Write list of Steam Library Directories to config file."""
        if not os.path.isdir(cfg_dir := os.path.dirname(self.file)):
            try:
                os.makedirs(cfg_dir)
            except OSError as exc:
                raise ConfigFileError(
                    "cannot create config dir", cfg_dir
                ) from exc
        with self._open("w") as cfg_file:
            lib_dirs.insert(0, CONFIG_PREFIX)
            cfg_file.write("\n".join(lib_dirs))

    @staticmethod
    def _get_lib_dir_list(lib_dirs: Union[str, List[str]]) -> List[str]:
        """Return lib_dirs as single element list if given as str."""
        if isinstance(lib_dirs, list):
            return lib_dirs
        return [lib_dirs]
